//
//  ViewController.swift
//  MetaLocatorSample
//
//  Created by Michael Fatica on 8/19/16.
//  Copyright © 2016 MetaLocator. All rights reserved.
//

import UIKit
import CoreLocation

class ViewController: UIViewController, UITableViewDelegate, UITableViewDataSource,CLLocationManagerDelegate {
    
    
    let defaultSession = NSURLSession(configuration: NSURLSessionConfiguration.defaultSessionConfiguration())
    var dataTask: NSURLSessionDataTask?
    
    // for tracking the user location
    var locationManager: CLLocationManager!
    
    // status variables to help track location manager state
    var locationErrorDetected : Bool = false
    var locationObtained : Bool = false
    var locationStatus : NSString = "Not Obtained"
    
    
    //this location is used to pass to the detail record
    var locationToPass:MetaLocations!


    //the location search input
    @IBOutlet weak var postalCodeTextInput: UITextField!
    
    //the search button
    @IBOutlet weak var searchButton: UIButton!
    
    // Data model: These MetaLocation objects will be the data for the table view cells
    var locations: [MetaLocations] = []
    
    // cell reuse id (cells that scroll out of view can be reused)
    let cellReuseIdentifier = "LocationsCellIdentifier"
    
    // don't forget to hook this up from the storyboard
    @IBOutlet var tableView: UITableView!
    
    //when the user clicks the search button, load the results
    @IBAction func searchButtonClick(sender: AnyObject) {
        self.loadLocations(self.postalCodeTextInput.text!)
    }
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        
        // Register the table view cell class and its reuse id
        self.tableView.registerClass(UITableViewCell.self, forCellReuseIdentifier: cellReuseIdentifier)
        
        // This view controller itself will provide the delegate methods and row data for the table view.
        tableView.delegate = self
        tableView.dataSource = self
        
        self.initializeManager()

    }
    
    
    /// initialize the CLLocationManager
    func initializeManager() {
        
        locationErrorDetected = false
        locationObtained = false
        locationManager = CLLocationManager()
        locationManager.delegate = self
        locationManager.desiredAccuracy = kCLLocationAccuracyBest
        locationManager.requestAlwaysAuthorization()
        
    }
    
    
    // handle location errors by terminated the location monitor
    func locationManager(manager: CLLocationManager, didFailWithError error: NSError) {
        
        locationManager.stopUpdatingLocation()
        
        if (locationErrorDetected == false) {
            locationErrorDetected = true
            print(error)
        }
    }
    
    // this function is called every time the user's location is updated.
    func locationManager(manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        let locValue:CLLocationCoordinate2D = manager.location!.coordinate
        
        if (locationObtained == false) {
            locationObtained = true

            //pass the comma-separated latitude/longitue to loadLocations
            let searchString: String = "\(locValue.latitude),\(locValue.longitude)"
            self.loadLocations(searchString)
            
        }
    }
    
    func locationManager(manager: CLLocationManager,
                         didChangeAuthorizationStatus status: CLAuthorizationStatus) {
        var shouldIAllow = false
        
        switch status {
        case CLAuthorizationStatus.Restricted:
            locationStatus = "Restricted Access to location"
        case CLAuthorizationStatus.Denied:
            locationStatus = "User denied access to location"
        case CLAuthorizationStatus.NotDetermined:
            locationStatus = "Status not determined"
        default:
            locationStatus = "Allowed to location Access"
            shouldIAllow = true
        }
        
        NSNotificationCenter.defaultCenter().postNotificationName("LabelHasbeenUpdated", object: nil)
        
        if (shouldIAllow == true) {
            
            NSLog("Location Permission Allowed")
            
            // Start location services
            locationManager.startUpdatingLocation()
            
        } else {
            
            NSLog("Denied access: \(locationStatus)")
        }
    }
    
    
    // number of rows in table view
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.locations.count
    }
    
    // create a cell for each table view row
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        //Use the SubTitle style so we can display the location name and distance
        let cell = UITableViewCell(style: UITableViewCellStyle.Subtitle, reuseIdentifier: cellReuseIdentifier)
   
        
        //set the cell label to teh location name
        cell.textLabel?.text = self.locations[indexPath.row].name
        
        //add the distance if we have it.
        cell.detailTextLabel?.text = self.locations[indexPath.row].getFormattedDistance()

        
        return cell
    }
    

    
    /// This function loads the locations around a provided `postalCode`
    /// :param: The postalCode to be searched around
    func loadLocations(postalCode: String) {
        
        //remove the existing items
        locations.removeAll()

        
        if dataTask != nil {
            dataTask?.cancel()
        }

        UIApplication.sharedApplication().networkActivityIndicatorVisible = true
        
        //Enter your Interface ID number for the Itemid
        let Itemid = 0000;
        
        //Set your MetaLocator APIKey, username and password
        let apikey = "----------------";
        var username = "----@----------.---";
        var password = "---------";
        
        //ensure safe handling of special characters in the username and password
        username = username.stringByAddingPercentEncodingWithAllowedCharacters(.URLHostAllowedCharacterSet())!
        
        password = password.stringByAddingPercentEncodingWithAllowedCharacters(.URLHostAllowedCharacterSet())!
        
        //the MetaLocator PaaS API Search function, with a hard-coded limit of 10 locations
        let url = NSURL(string: "https://code.metalocator.com/webapi/api/search?Itemid=\(Itemid)&apikey=\(apikey)&username=\(username)&password=\(password)&postal_code=\(postalCode)&limit=10");
        
        
        //var urlString: String = url!.absoluteString
        
        //
        dataTask = defaultSession.dataTaskWithURL(url!) {
            data, response, error in
            //
            dispatch_async(dispatch_get_main_queue()) {
                UIApplication.sharedApplication().networkActivityIndicatorVisible = false
            }
            //
            if let error = error {
                print(error.localizedDescription)
            } else if let httpResponse = response as? NSHTTPURLResponse {
                if httpResponse.statusCode == 200 {
                    
                    self.updateLocations(data)
                }
            }
        }
        //
        dataTask?.resume()
        
        
    }
    
    /// This function takes the API results, parses the JSON and creates a series of MetaLocation objects
    /// :data: The raw JSON text of the MetaLocator API Request.
    func updateLocations(data: NSData?) {

        do {
            
            let json = try NSJSONSerialization.JSONObjectWithData(data!, options: .AllowFragments)
            
            
            if let json = json as? NSArray {
                
                for jsonRow in json {
                    
                    //if the name key has a string value
                    if let name = jsonRow.valueForKey("name") as? String {
                        
                        //get the distance as a Double
                        let distance = jsonRow.valueForKey("Distance")?.doubleValue
                        
                        //pull other attributes as optional strings
                        let description = jsonRow.valueForKey("description") as? String
                        
                        let image = jsonRow.valueForKey("image") as? String
                        
                        let link = jsonRow.valueForKey("link") as? String
                        
                        
                        let address = jsonRow.valueForKey("address") as? String
                        let address2 = jsonRow.valueForKey("address2") as? String
                        let city = jsonRow.valueForKey("city") as? String
                        let state = jsonRow.valueForKey("state") as? String
                        let postalcode = jsonRow.valueForKey("postalcode") as? String
                        let country = jsonRow.valueForKey("country") as? String
                        let phone = jsonRow.valueForKey("phone") as? String
                        let email = jsonRow.valueForKey("email") as? String
                    

                        // cretae a new MetaLocations object with this name and distance
                        let location = MetaLocations(name: name, desc: description, distance: distance, image:image, link: link, address: address,address2: address2,city: city,state: state,postalcode: postalcode, country: country,phone: phone,email: email)
                        
                        //add it to the views list of locations
                        self.locations.append(location);
                        
                        
                    }
                    
                }
            }
            
        } catch {
            print("error serializing JSON: \(error)")
        }
        
        
        dispatch_async(dispatch_get_main_queue()) {
            
            //queue a refresh of the table view data
            self.tableView.reloadData()
            
        }
    }
    
    // method to run when table view cell is tapped
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        tableView.deselectRowAtIndexPath(indexPath, animated: true)
        
        //initialize a local MetaLocations object to the object at the chosen row index
        let loc = self.locations[indexPath.row] as MetaLocations
        
        //set our controllers locationToPass as this location
        self.locationToPass = loc
        
        //perform the segue
        performSegueWithIdentifier("showLocationDetailSegue", sender: self)
        
        
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?){
        
        if (segue.identifier == "showLocationDetailSegue") {
            
            // initialize new view controller and cast it as your view controller
            let viewController = segue.destinationViewController as! LocationDetailViewController
            
            // your new view controller should have property that will store passed value
            viewController.receivedLocation = self.locationToPass
            
        }
        
    }
}